@@ -2,7 +2,7 @@
 * dvbinput.cpp: DVB streams
 *-------------------------------------------------------------------------------
 * (c)1999-2002 VideoLAN
-* $Id: dvbinput.cpp,v 1.21.4.4 2003/05/20 21:24:04 nitrox Exp $
+* $Id: dvbinput.cpp,v 1.21.4.5 2003/11/25 14:29:09 nitrox Exp $
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *          Damien Lucas <nitrox@videolan.org>
@@ -151,7 +151,7 @@ void C_DvbInput::OnInit()
   {
     strncpy(filen, dvbrc.GetString(), dvbrc.Length()+1);
   }
-  dvb->init("", "", 0, iNumber);
+  dvb->init("", "", iNumber, 0);
 
   sprintf(devname, DVR_DEV,   iNumber, 0); m_strDVR   = C_String(devname);
   sprintf(devname, DEMUX_DEV, iNumber, 0); m_strDEMUX = C_String(devname);
