@@ -2,7 +2,7 @@
 * localinput.cpp: Local streams
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: localinput.cpp,v 1.13 2002/12/09 01:36:24 nitrox Exp $
+* $Id: localinput.cpp,v 1.13.4.1 2003/06/04 19:06:14 alexis Exp $
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
@@ -229,6 +229,10 @@ void C_LocalInput::OnStartStreaming(C_Broadcast* pBroadcast)
     strReaderType = "dvd";
     C_String strDevice = pApp->GetSetting(strId+".Device", "");
     pBroadcast->SetOption("device", strDevice);
+    C_String strDvdTitle = pApp->GetSetting(strId+".dvdtitle","");
+    pBroadcast->SetOption("dvdtitle", strDvdTitle);
+    C_String strDvdChapter = pApp->GetSetting(strId+".dvdchapter","");
+    pBroadcast->SetOption("dvdchapter", strDvdChapter);
 
     // Converter configuration
     strConverterType = "ps2ts";
