@@ -2,10 +2,11 @@
 * ps2ts.cpp: MPEG1 and MPEG2 PS to MPEG2 TS converter
 *-------------------------------------------------------------------------------
 * (c)1999-2001 VideoLAN
-* $Id: ps2ts.cpp,v 1.4.4.1 2003/02/14 19:57:55 nitrox Exp $
+* $Id: ps2ts.cpp,v 1.4.4.2 2003/06/24 23:56:56 nitrox Exp $
 *
 * Authors: Benoit Steiner <benny@via.ecp.fr>
 *          Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
+*          Damien Lucas <damien.lucas@anevia.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
@@ -469,8 +470,10 @@ template <class Reader, class TsProvider>
 template <class Reader, class TsProvider>
    int C_Ps2Ts<Reader, TsProvider>::ParsePES(C_TsPacket* pPacket, u8* pPosInTs)
 {
+    byte pBuff[188];
   ASSERT(pPacket);
   ASSERT(pPosInTs);
+  unsigned int iDataLen_old;
 
   // Pour le debug uniquement
 //  if(m_bPESStart)
@@ -518,9 +521,9 @@ template <class Reader, class TsProvider>
       }
       }
     }
-    
+
     // Write the beginning of the PES packet (corresponds to the look ahead)
-    byte* pTsPayload = (byte*)(*pPacket);
+    byte* pTsPayload = (byte*)((*pPacket));
     SET_U32_TO(pTsPayload[iPos], m_iDataType);
     SET_U16_TO(pTsPayload[iPos+START_CODE_LEN], m_iDataLen);
     iPos += START_CODE_LEN + PES_SIZE_LEN;
@@ -545,40 +548,70 @@ template <class Reader, class TsProvider>
     // Fill the Ts packet
     if(m_iDataLen >= TS_PACKET_LEN - iPos)
     {
-      // Fill the remaining payload of the TS packet
-      m_iStatus = m_pReader->Read(((byte*)(*pPacket)+iPos), TS_PACKET_LEN-iPos);
-      ASSERT(m_iStatus >= 0);
-      m_iStatus = (m_iStatus != TS_PACKET_LEN-iPos);
-
-      m_iDataLen -= (TS_PACKET_LEN-iPos);
-
-      //-----------------
-      if(m_bPESStart)
+      if(*((byte*)(*pPacket) + iPos - 3) == PES_ID_PRIVATE_1 && m_bPESStart)
       {
-      if(*((byte*)(*pPacket) + iPos - 3) == PES_ID_PRIVATE_1)
-      {
-//      printf("Id : 0x%x", *((byte*)(*pPacket) + iPos - 3));
-        u8 iOffset = *((byte*)(*pPacket) + iPos + 2);
-//        printf(" Offset : 0x%x", iOffset);
-        u8 iPrivateId = *((byte*)(*pPacket) + iPos + 3 + iOffset);
-//        printf(" PrivateId : 0x%x", iPrivateId);
-/*        if((iPrivateId & 0xf0) == 0x80)
-        {*/
-          m_pCurrentData = m_cPgrmDescriptor.GetDescriptor(m_iDataType & 0xFF,
-                                                           iPrivateId);
+          // Read in a buffer
+          m_iStatus = m_pReader->Read(pBuff, TS_PACKET_LEN-iPos);
+          ASSERT(m_iStatus >= 0);
+          m_iStatus = (m_iStatus != TS_PACKET_LEN-iPos);
+          iDataLen_old = m_iDataLen;
+          m_iDataLen -= (TS_PACKET_LEN-iPos);
+
+          u8 iOffset =  pBuff[2];
+          u8 iPrivateId = pBuff[3 + iOffset];
+          byte* pTsPayload;
+          m_pCurrentData = m_cPgrmDescriptor.GetDescriptor(
+                                            m_iDataType & 0xFF, iPrivateId);
           u16 iPid =  m_pCurrentData->GetPid();
-          u16 iData = U16_AT(*((byte*)(pPacket) + 1));
-//          printf(" Data : 0x%x", iData);
-          iPid = (iPid & 0x1fff) | (iData & 0xe000);
-//          printf(" NewPid : 0x%x", iPid);
-          SET_U16_TO(*((byte*)(pPacket) + 1), iPid);
-          *((byte*)(pPacket) + 3) = (m_pCurrentData->GetCounter() & 0x0f) |
+          u16 iData = U16_AT(*(pPacket + 1));
+          switch(iPrivateId)
+          {
+              case 0x80: //A52
+                  // Add 4 bytes stuffing in the TS packet
+                  // We have already written the PES start code and the PES
+                  // length at the beginning of the TS packet but they will be
+                  // erased by the stuffing bytes: we must rewrite them
+                  iPos = pPacket->AddStuffingBytes(TS_PACKET_LEN+2-iPos);
+                  pTsPayload = (byte*)(*pPacket);
+                  SET_U32_TO(pTsPayload[iPos], m_iDataType);
+                  SET_U16_TO(pTsPayload[iPos+START_CODE_LEN], iDataLen_old -4);
+                  iPos += START_CODE_LEN+PES_SIZE_LEN;
+
+                  //Copy the PES header
+                  memcpy((byte*)(*pPacket)+iPos, pBuff, iOffset+3);
+                  //Copy the A52 datas
+                  memcpy((byte*)(*pPacket)+iPos+iOffset +3,
+                            pBuff +iOffset + 4 + 3,
+                                                TS_PACKET_LEN-iPos-iOffset -3);
+
+                  iPid = (iPid & 0x1fff) | (iData & 0xe000);
+                  SET_U16_TO(*((byte*)(pPacket) + 1), iPid);
+                  *((byte*)(pPacket) + 3) =
+                                    (m_pCurrentData->GetCounter() & 0x0f) |
                                     (*((byte*)(pPacket) + 3) & 0xf0);
-/*        }*/
-//      printf("\n");
+                  break;
+
+              default:
+                  memcpy((byte*)(*pPacket)+iPos, pBuff, TS_PACKET_LEN-iPos);
+                  iPid = (iPid & 0x1fff) | (iData & 0xe000);
+                  SET_U16_TO(*((byte*)(pPacket) + 1), iPid);
+                  *((byte*)(pPacket) + 3) =
+                                    (m_pCurrentData->GetCounter() & 0x0f) |
+                                    (*((byte*)(pPacket) + 3) & 0xf0);
+                  break;
+          }
       }
+      else
+      {
+        // Fill the remaining payload of the TS packet
+        m_iStatus = m_pReader->Read(((byte*)(*pPacket)+iPos),
+                                                            TS_PACKET_LEN-iPos);
+        ASSERT(m_iStatus >= 0);
+        m_iStatus = (m_iStatus != TS_PACKET_LEN-iPos);
+
+        m_iDataLen -= (TS_PACKET_LEN-iPos);
+
       }
-      //-----------------
     }
     else
     {
@@ -605,11 +638,11 @@ template <class Reader, class TsProvider>
         // Simply add the stuffing bytes at the end of the TS header
         iPos = pPacket->AddStuffingBytes(m_iDataLen);
       }
-      
+
       m_iStatus = m_pReader->Read(((byte*)(*pPacket)+iPos), m_iDataLen);
       ASSERT(m_iStatus >= 0);
       m_iStatus = (m_iStatus != m_iDataLen);
-        
+
       //-----------------
       if(m_bPESStart)
       {
@@ -633,7 +666,6 @@ template <class Reader, class TsProvider>
           *((byte*)(pPacket) + 3) = (m_pCurrentData->GetCounter() & 0x0f) |
                                     (*((byte*)(pPacket) + 3) & 0xf0);
 /*        }*/
-//      printf("\n");
       }
       }
       //-----------------
